function varargout=refine(Oh,ndiscells,types)
% Oh: OrthMesh object (type: 'orthotope')
% ndiscells: d-by-nme array  or 1-by-nme array
%   ndiscells(:,k) numbers of discretisation in each direction
% types: 1-by-nme array, 0 for simplicial, 1 for orthotope
  assert( strcmp(class(Oh),'OrthMesh') , '1st parameter must be an OrthMesh object')
  assert( strcmp(Oh.type,'orthotope') )
  q=Oh.Mesh.q;me=Oh.Mesh.me;nme=Oh.Mesh.nme;
  if nargin==2,types='orthotope';end
  nout=2;
  if ischar(types)
    nout=1;
    stype=types;
    if strcmp(types,'simplicial'),types=zeros(1,nme);else,types=ones(1,nme);end
  end
  if nout==2
    scsimp=subcells_type(q,me,ndiscells,types,'simplicial');
    scorth=subcells_type(q,me,ndiscells,types,'orthotope');
    if nargout==2
      varargout={scsimp,scorth};
    else
      if ~isempty(scsimp)
        varargout{1}={scsimp{:},scorth{:}};
      else
        varargout{1}={scorth{:}};
      end
    end
  else
    varargout{1}=subcells_type(q,me,ndiscells,types,stype);
  end
end

function scells=subcells_type(q,me,ndiscells,types,type)
  if ~fc_tools.comp.isOctave && verLessThan('matlab','R2016b')
    trans=@(n,ncol) repmat(n,[1,ncol]);
  else
    trans=@(n,ncol) n;
  end
  if strcmp(type,'simplicial')
    idxcells=find(types==0);
  else
    idxcells=find(types==1);
  end
  nme=size(me,2);
  if size(ndiscells,1)==1
    ndiscells=repmat(ndiscells,[size(q,1),1]);
  end
  cells=ndiscells(:,idxcells);
  i=1;
  scells={};
  N=[];
  while ~isempty(cells)
    n=cells(:,1);
    N=[N,n];
    idx=find( all(cells==trans(n,size(cells,2))));
    Is=idxcells(idx);
    scs=cells(:,idx);
    Me=me(:,Is);
    [iq,ic,ia]=unique(Me(:)); 
    Q=q(:,iq);
    ME=reshape(ia,size(Me));
    [Qsub,MEsub]=fc_hypermesh.refinement.buildsubcells(Q,ME,n',type);
    scells{i}=struct('type',type,'d',size(Qsub,1),'n',n','q',Qsub,'me',MEsub,'nq',size(Qsub,2),'nme',size(MEsub,2),'ncell',Is);
    
    cidx=setdiff(1:size(cells,2),idx);
    cells=cells(:,cidx);
    idxcells=idxcells(cidx);
    i=i+1;
  end
  %[B,idx]=sortrows(N','descend');
  [B,idx]=sortrows(N');
  scells={scells{idx}};
end
